@@ -14,7 +14,7 @@ NULL
 #'
 #' @note When called, the returned function attempts to preserve the R session's
 #'   current seed by snapshotting and restoring
-#'   \code{\link[base]{.Random.seed}}.
+#'   [base::.Random.seed()].
 #'
 #' @examples
 #' rnormA <- repeatable(rnorm)
@@ -173,12 +173,12 @@ anyUnnamed <- function(x) {
 }
 
 
-# Given a vector/list, returns a named vector (the labels will be blank).
-asNamedVector <- function(x) {
-  if (!is.null(names(x)))
-    return(x)
+# Given a vector/list, returns a named vector/list (the labels will be blank).
+asNamed <- function(x) {
+  if (is.null(names(x))) {
+    names(x) <- character(length(x))
+  }
 
-  names(x) <- rep.int("", length(x))
   x
 }
 
@@ -468,12 +468,12 @@ exprToFunction <- function(expr, env=parent.frame(), quoted=FALSE) {
 #' Installs an expression in the given environment as a function, and registers
 #' debug hooks so that breakpoints may be set in the function.
 #'
-#' This function can replace \code{exprToFunction} as follows: we may use
-#' \code{func <- exprToFunction(expr)} if we do not want the debug hooks, or
-#' \code{installExprFunction(expr, "func")} if we do. Both approaches create a
-#' function named \code{func} in the current environment.
+#' This function can replace `exprToFunction` as follows: we may use
+#' `func <- exprToFunction(expr)` if we do not want the debug hooks, or
+#' `installExprFunction(expr, "func")` if we do. Both approaches create a
+#' function named `func` in the current environment.
 #'
-#' @seealso Wraps \code{\link{exprToFunction}}; see that method's documentation
+#' @seealso Wraps [exprToFunction()]; see that method's documentation
 #'   for more documentation and examples.
 #'
 #' @param expr A quoted or unquoted expression
@@ -485,7 +485,7 @@ exprToFunction <- function(expr, env=parent.frame(), quoted=FALSE) {
 #' @param label A label for the object to be shown in the debugger. Defaults to
 #'   the name of the calling function.
 #' @param wrappedWithLabel,..stacktraceon Advanced use only. For stack manipulation purposes; see
-#'   \code{\link{stacktrace}}.
+#'   [stacktrace()].
 #' @export
 installExprFunction <- function(expr, name, eval.env = parent.frame(2),
                                 quoted = FALSE,
@@ -516,6 +516,7 @@ installExprFunction <- function(expr, name, eval.env = parent.frame(2),
 #'
 #' Returns a named list of key-value pairs.
 #'
+#' @noMd
 #' @param str The query string. It can have a leading \code{"?"} or not.
 #' @param nested Whether to parse the query string of as a nested list when it
 #'   contains pairs of square brackets \code{[]}. For example, the query
@@ -613,7 +614,7 @@ shinyCallingHandlers <- function(expr) {
 
 #' Print message for deprecated functions in Shiny
 #'
-#' To disable these messages, use \code{options(shiny.deprecation.messages=FALSE)}.
+#' To disable these messages, use `options(shiny.deprecation.messages=FALSE)`.
 #'
 #' @param new Name of replacement function.
 #' @param msg Message to print. If used, this will override the default message.
@@ -949,28 +950,28 @@ columnToRowData <- function(data) {
 #'
 #' This should be used when you want to let the user see an error
 #' message even if the default is to sanitize all errors. If you have an
-#' error \code{e} and call \code{stop(safeError(e))}, then Shiny will
-#' ignore the value of \code{getOption("shiny.sanitize.errors")} and always
+#' error `e` and call `stop(safeError(e))`, then Shiny will
+#' ignore the value of `getOption("shiny.sanitize.errors")` and always
 #' display the error in the app itself.
 #'
 #' @param error Either an "error" object or a "character" object (string).
 #' In the latter case, the string will become the message of the error
-#' returned by \code{safeError}.
+#' returned by `safeError`.
 #'
 #' @return An "error" object
 #'
-#' @details An error generated by \code{safeError} has priority over all
+#' @details An error generated by `safeError` has priority over all
 #' other Shiny errors. This can be dangerous. For example, if you have set
-#' \code{options(shiny.sanitize.errors = TRUE)}, then by default all error
+#' `options(shiny.sanitize.errors = TRUE)`, then by default all error
 #' messages are omitted in the app, and replaced by a generic error message.
-#' However, this does not apply to \code{safeError}: whatever you pass
-#' through \code{error} will be displayed to the user. So, this should only
+#' However, this does not apply to `safeError`: whatever you pass
+#' through `error` will be displayed to the user. So, this should only
 #' be used when you are sure that your error message does not contain any
-#' sensitive information. In those situations, \code{safeError} can make
+#' sensitive information. In those situations, `safeError` can make
 #' your users' lives much easier by giving them a hint as to where the
 #' error occurred.
 #'
-#' @seealso \code{\link{shiny-options}}
+#' @seealso [shiny-options()]
 #'
 #' @examples
 #' ## Only run examples in interactive R sessions
@@ -1084,58 +1085,58 @@ reactiveStop <- function(message = "", class = NULL) {
 
 #' Validate input values and other conditions
 #'
-#' For an output rendering function (e.g. \code{\link{renderPlot}()}), you may
+#' For an output rendering function (e.g. [renderPlot()]), you may
 #' need to check that certain input values are available and valid before you
-#' can render the output. \code{validate} gives you a convenient mechanism for
+#' can render the output. `validate` gives you a convenient mechanism for
 #' doing so.
 #'
-#' The \code{validate} function takes any number of (unnamed) arguments, each of
+#' The `validate` function takes any number of (unnamed) arguments, each of
 #' which represents a condition to test. If any of the conditions represent
 #' failure, then a special type of error is signaled which stops execution. If
 #' this error is not handled by application-specific code, it is displayed to
 #' the user by Shiny.
 #'
-#' An easy way to provide arguments to \code{validate} is to use the \code{need}
+#' An easy way to provide arguments to `validate` is to use the `need`
 #' function, which takes an expression and a string; if the expression is
 #' considered a failure, then the string will be used as the error message. The
-#' \code{need} function considers its expression to be a failure if it is any of
+#' `need` function considers its expression to be a failure if it is any of
 #' the following:
 #'
 #' \itemize{
-#'   \item{\code{FALSE}}
-#'   \item{\code{NULL}}
-#'   \item{\code{""}}
+#'   \item{`FALSE`}
+#'   \item{`NULL`}
+#'   \item{`""`}
 #'   \item{An empty atomic vector}
 #'   \item{An atomic vector that contains only missing values}
-#'   \item{A logical vector that contains all \code{FALSE} or missing values}
-#'   \item{An object of class \code{"try-error"}}
-#'   \item{A value that represents an unclicked \code{\link{actionButton}}}
+#'   \item{A logical vector that contains all `FALSE` or missing values}
+#'   \item{An object of class `"try-error"`}
+#'   \item{A value that represents an unclicked [actionButton()]}
 #' }
 #'
 #' If any of these values happen to be valid, you can explicitly turn them to
-#' logical values. For example, if you allow \code{NA} but not \code{NULL}, you
-#' can use the condition \code{!is.null(input$foo)}, because \code{!is.null(NA)
-#' == TRUE}.
+#' logical values. For example, if you allow `NA` but not `NULL`, you
+#' can use the condition `!is.null(input$foo)`, because `!is.null(NA)
+#' == TRUE`.
 #'
-#' If you need validation logic that differs significantly from \code{need}, you
+#' If you need validation logic that differs significantly from `need`, you
 #' can create other validation test functions. A passing test should return
-#' \code{NULL}. A failing test should return an error message as a
+#' `NULL`. A failing test should return an error message as a
 #' single-element character vector, or if the failure should happen silently,
-#' \code{FALSE}.
+#' `FALSE`.
 #'
 #' Because validation failure is signaled as an error, you can use
-#' \code{validate} in reactive expressions, and validation failures will
+#' `validate` in reactive expressions, and validation failures will
 #' automatically propagate to outputs that use the reactive expression. In
-#' other words, if reactive expression \code{a} needs \code{input$x}, and two
-#' outputs use \code{a} (and thus depend indirectly on \code{input$x}), it's
-#' not necessary for the outputs to validate \code{input$x} explicitly, as long
-#' as \code{a} does validate it.
+#' other words, if reactive expression `a` needs `input$x`, and two
+#' outputs use `a` (and thus depend indirectly on `input$x`), it's
+#' not necessary for the outputs to validate `input$x` explicitly, as long
+#' as `a` does validate it.
 #'
-#' @param ... A list of tests. Each test should equal \code{NULL} for success,
-#'   \code{FALSE} for silent failure, or a string for failure with an error
+#' @param ... A list of tests. Each test should equal `NULL` for success,
+#'   `FALSE` for silent failure, or a string for failure with an error
 #'   message.
 #' @param errorClass A CSS class to apply. The actual CSS string will have
-#'   \code{shiny-output-error-} prepended to this value.
+#'   `shiny-output-error-` prepended to this value.
 #' @export
 #' @examples
 #' ## Only run examples in interactive R sessions
@@ -1187,10 +1188,10 @@ validate <- function(..., errorClass = character(0)) {
 #' @param expr An expression to test. The condition will pass if the expression
 #'   meets the conditions spelled out in Details.
 #' @param message A message to convey to the user if the validation condition is
-#'   not met. If no message is provided, one will be created using \code{label}.
-#'   To fail with no message, use \code{FALSE} for the message.
+#'   not met. If no message is provided, one will be created using `label`.
+#'   To fail with no message, use `FALSE` for the message.
 #' @param label A human-readable name for the field that may be missing. This
-#'   parameter is not needed if \code{message} is provided, but must be provided
+#'   parameter is not needed if `message` is provided, but must be provided
 #'   otherwise.
 #' @export
 #' @rdname validate
@@ -1211,7 +1212,7 @@ need <- function(expr, message = paste(label, "must be provided"), label) {
 #' operation is stopped by raising a "silent" exception (not logged by Shiny,
 #' nor displayed in the Shiny app's UI).
 #'
-#' The \code{req} function was designed to be used in one of two ways. The first
+#' The `req` function was designed to be used in one of two ways. The first
 #' is to call it like a statement (ignoring its return value) before attempting
 #' operations using the required values:
 #'
@@ -1221,9 +1222,9 @@ need <- function(expr, message = paste(label, "must be provided"), label) {
 #'   # Code that uses input$a, input$b, and/or rv$state...
 #' })}
 #'
-#' In this example, if \code{r()} is called and any of \code{input$a},
-#' \code{input$b}, and \code{rv$state} are \code{NULL}, \code{FALSE}, \code{""},
-#' etc., then the \code{req} call will trigger an error that propagates all the
+#' In this example, if `r()` is called and any of `input$a`,
+#' `input$b`, and `rv$state` are `NULL`, `FALSE`, `""`,
+#' etc., then the `req` call will trigger an error that propagates all the
 #' way up to whatever render block or observer is executing.
 #'
 #' The second is to use it to wrap an expression that must be truthy:
@@ -1236,70 +1237,70 @@ need <- function(expr, message = paste(label, "must be provided"), label) {
 #'   }
 #' })}
 #'
-#' In this example, \code{req(input$plotType)} first checks that
-#' \code{input$plotType} is truthy, and if so, returns it. This is a convenient
+#' In this example, `req(input$plotType)` first checks that
+#' `input$plotType` is truthy, and if so, returns it. This is a convenient
 #' way to check for a value "inline" with its first use.
 #'
-#' \strong{Truthy and falsy values}
+#' **Truthy and falsy values**
 #'
 #' The terms "truthy" and "falsy" generally indicate whether a value, when
-#' coerced to a \code{\link[base]{logical}}, is \code{TRUE} or \code{FALSE}. We use
+#' coerced to a [base::logical()], is `TRUE` or `FALSE`. We use
 #' the term a little loosely here; our usage tries to match the intuitive
 #' notions of "Is this value missing or available?", or "Has the user provided
 #' an answer?", or in the case of action buttons, "Has the button been
 #' clicked?".
 #'
-#' For example, a \code{textInput} that has not been filled out by the user has
-#' a value of \code{""}, so that is considered a falsy value.
+#' For example, a `textInput` that has not been filled out by the user has
+#' a value of `""`, so that is considered a falsy value.
 #'
-#' To be precise, \code{req} considers a value truthy \emph{unless} it is one
+#' To be precise, `req` considers a value truthy *unless* it is one
 #' of:
 #'
 #' \itemize{
-#'   \item{\code{FALSE}}
-#'   \item{\code{NULL}}
-#'   \item{\code{""}}
+#'   \item{`FALSE`}
+#'   \item{`NULL`}
+#'   \item{`""`}
 #'   \item{An empty atomic vector}
 #'   \item{An atomic vector that contains only missing values}
-#'   \item{A logical vector that contains all \code{FALSE} or missing values}
-#'   \item{An object of class \code{"try-error"}}
-#'   \item{A value that represents an unclicked \code{\link{actionButton}}}
+#'   \item{A logical vector that contains all `FALSE` or missing values}
+#'   \item{An object of class `"try-error"`}
+#'   \item{A value that represents an unclicked [actionButton()]}
 #' }
 #'
-#' Note in particular that the value \code{0} is considered truthy, even though
-#' \code{as.logical(0)} is \code{FALSE}.
+#' Note in particular that the value `0` is considered truthy, even though
+#' `as.logical(0)` is `FALSE`.
 #'
 #' If the built-in rules for truthiness do not match your requirements, you can
-#' always work around them. Since \code{FALSE} is falsy, you can simply provide
-#' the results of your own checks to \code{req}:
+#' always work around them. Since `FALSE` is falsy, you can simply provide
+#' the results of your own checks to `req`:
 #'
-#' \code{req(input$a != 0)}
+#' `req(input$a != 0)`
 #'
-#' \strong{Using \code{req(FALSE)}}
+#' **Using `req(FALSE)`**
 #'
-#' You can use \code{req(FALSE)} (i.e. no condition) if you've already performed
+#' You can use `req(FALSE)` (i.e. no condition) if you've already performed
 #' all the checks you needed to by that point and just want to stop the reactive
 #' chain now. There is no advantange to this, except perhaps ease of readibility
 #' if you have a complicated condition to check for (or perhaps if you'd like to
-#' divide your condition into nested \code{if} statements).
+#' divide your condition into nested `if` statements).
 #'
-#' \strong{Using \code{cancelOutput = TRUE}}
+#' **Using `cancelOutput = TRUE`**
 #'
-#' When \code{req(..., cancelOutput = TRUE)} is used, the "silent" exception is
+#' When `req(..., cancelOutput = TRUE)` is used, the "silent" exception is
 #' also raised, but it is treated slightly differently if one or more outputs are
 #' currently being evaluated. In those cases, the reactive chain does not proceed
 #' or update, but the output(s) are left is whatever state they happen to be in
 #' (whatever was their last valid state).
 #'
 #' Note that this is always going to be the case if
-#' this is used inside an output context (e.g. \code{output$txt <- ...}). It may
+#' this is used inside an output context (e.g. `output$txt <- ...`). It may
 #' or may not be the case if it is used inside a non-output context (e.g.
-#' \code{\link{reactive}}, \code{\link{observe}} or \code{\link{observeEvent}})
-#' -- depending on whether or not there is an \code{output$...} that is triggered
+#' [reactive()], [observe()] or [observeEvent()])
+#' --- depending on whether or not there is an `output$...` that is triggered
 #' as a result of those calls. See the examples below for concrete scenarios.
 #'
 #' @param ... Values to check for truthiness.
-#' @param cancelOutput If \code{TRUE} and an output is being evaluated, stop
+#' @param cancelOutput If `TRUE` and an output is being evaluated, stop
 #'   processing as usual but instead of clearing the output, leave it in
 #'   whatever state it happens to be in.
 #' @param x An expression whose truthiness value we want to determine
@@ -1438,7 +1439,7 @@ stopWithCondition <- function(class, message) {
 #' This function returns the information about the current Shiny Server, such as
 #' its version, and whether it is the open source edition or professional
 #' edition. If the app is not served through the Shiny Server, this function
-#' just returns \code{list(shinyServer = FALSE)}.
+#' just returns `list(shinyServer = FALSE)`.
 #'
 #' This function will only return meaningful data when using Shiny Server
 #' version 1.2.2 or later.
@@ -1560,6 +1561,25 @@ URLencode <- function(value, reserved = FALSE) {
   if (reserved) encodeURIComponent(value) else encodeURI(value)
 }
 
+# Make user-supplied dates are either NULL or can be coerced
+# to a yyyy-mm-dd formatted string. If a date is specified, this
+# function returns a string for consistency across locales.
+# Also, `as.Date()` is used to coerce strings to date objects
+# so that strings like "2016-08-9" are expanded to "2016-08-09"
+dateYMD <- function(date = NULL, argName = "value") {
+  if (!length(date)) return(NULL)
+  if (length(date) > 1) warning("Expected `", argName, "` to be of length 1.")
+  tryCatch(date <- format(as.Date(date), "%Y-%m-%d"),
+    error = function(e) {
+      warning(
+        "Couldn't coerce the `", argName,
+        "` argument to a date string with format yyyy-mm-dd",
+        call. = FALSE
+      )
+    }
+  )
+  date
+}
 
 # This function takes a name and function, and it wraps that function in a new
 # function which calls the original function using the specified name. This can
@@ -1730,6 +1750,7 @@ createVarPromiseDomain <- function(env, name, value) {
 
 getSliderType <- function(min, max, value) {
   vals <- dropNulls(list(value, min, max))
+  if (length(vals) == 0) return("")
   type <- unique(lapply(vals, function(x) {
     if      (inherits(x, "Date"))   "date"
     else if (inherits(x, "POSIXt")) "datetime"
@@ -1779,3 +1800,8 @@ constantTimeEquals <- function(raw1, raw2) {
 
   sum(as.integer(xor(raw1, raw2))) == 0
 }
+
+cat_line <- function(...) {
+  cat(paste(..., "\n", collapse = ""))
+}
+
